/* Document preview */

img.preview {
  display: block;
  border: 1px black;
  margin: 0px auto;
}

.preview-container {
  position: relative;
  
  .preview-nav-button {
    position: absolute;
    top: 0px;
    left: 0px;
    bottom: 0px;
    display: block;
    z-index: 100;
    width: 15%;
    text-align: center;
    color: transparent;
    opacity: 0.9;
    
    &:hover {
      color: #0c3b3d;      
      cursor: pointer;
    }

    .fa {
      display: inline-block;
      position: absolute;
      top: 50%;
    }
  }  /* .preview-nav */
  
  .preview-prev .fa {
      left: 50%;
  }
  .preview-next {
    right: 0px;
    left: auto;
    
    .fa {
      right: 50%;
    }
  }
}

/* folder / documents objects table */
table#objects-table tbody td > img + i,
.folder-items li > img + i {
  position: relative;
  left: -14px;
  font-size: 9px;
}

/* My own (ugly) boxes */
.property {
  &-box .nav > li > a {
    display: inline;
    background-color: white;
  }

  &-list {
    padding-left: 14px;
    padding-right: 14px;
    margin-left: 0;
    margin-bottom: 5px;
    list-style: none;

    li {
      margin-top: 5px;
      margin-bottom: 5px;
    }
  }
}

#objects-table_wrapper img {
  border-radius: 20%;
}

/* File Uploads (currently not used) */
.btn.btn-file {
  overflow: hidden;
  position: relative;
  vertical-align: middle;

  > input[type="file"] {
    width: 100%;
    position: absolute;
    left: 0;
    top: 0;
    opacity: 0;
    cursor: pointer;
  }
}

.fileupload {
  margin-bottom: 9px;

  .uneditable-input {
    display: inline-block;
    margin-bottom: 0;
    vertical-align: middle;
  }

  .thumbnail {
    overflow: hidden;
    display: inline-block;
    margin-bottom: 5px;
    vertical-align: middle;
    text-align: center;
  }

  .thumbnail > img {
    display: inline-block;
    vertical-align: middle;
    max-height: 100%;
  }

  .btn {
    vertical-align: middle;
  }

  &-exists &-new,
  &-new &-exists {
    display: none;
  }

  &-inline &-controls {
    display: inline;
  }

  &-new .input-append .btn-file {
    -webkit-border-radius: 0 3px 3px 0;
    -moz-border-radius: 0 3px 3px 0;
    border-radius: 0 3px 3px 0;
  }
}
