<html>
<body>

<h1>Veuillez confirmer votre demande de désabonnement des notifications</h1>

<p>
Vous semblez vouloir vous désabonner des notifications quotidiennes des
  communautés.
</p>

<p>
  Pour confirmer cette décision, merci de bien vouloir cliquer sur le bouton
  ci-dessous:
</p>

<form action="." method="POST">
  <button>Me désabonner</button>
</form>

</body>
</html>